@@ -140,8 +140,21 @@ class PipVersionAction(Action):
         sys.exit(0)
 
 
-def fill_common(subparser, path=True):
+def fill_common(subparser):
     group = subparser.add_argument_group("language")
+    group.add_argument(
+        "-p",
+        "--path",
+        metavar="DIR",
+        type=str,
+        action="append",
+        help="local directory to search for imports (can supply multiple times)",
+    )
+    group.add_argument(
+        "--no-outside-imports",
+        action="store_true",
+        help="deny local imports from outside directory of main WDL file (or --path)",
+    )
     group.add_argument(
         "--no-quant-check",
         dest="check_quant",
@@ -151,15 +164,6 @@ def fill_common(subparser, path=True):
             "backwards compatibility with older WDL)"
         ),
     )
-    if path:
-        group.add_argument(
-            "-p",
-            "--path",
-            metavar="DIR",
-            type=str,
-            action="append",
-            help="local directory to search for imports",
-        )
     group = subparser.add_argument_group("debugging")
     group.add_argument(
         "--debug", action="store_true", help="maximally verbose logging & exception tracebacks"
@@ -216,6 +220,7 @@ def check(
     show_all=False,
     suppress=None,
     shellcheck=True,
+    no_outside_imports=False,
     **kwargs,
 ):
     from . import Lint
@@ -231,7 +236,12 @@ def check(
     shown = [0]
     for uri1 in uri or []:
         try:
-            doc = load(uri1, path or [], check_quant=check_quant, read_source=read_source)
+            doc = load(
+                uri1,
+                path or [],
+                check_quant=check_quant,
+                read_source=make_read_source(no_outside_imports),
+            )
         except (Error.SyntaxError, Error.ValidationError, Error.MultipleValidationErrors) as exn:
             if not getattr(exn, "declared_wdl_version", None):
                 atexit.register(
@@ -390,23 +400,44 @@ def print_error(exn):
                 quant_warning = True
 
 
-async def read_source(uri, path, importer):
-    from urllib import parse, request
+def make_read_source(no_outside_imports):
+    top_dir = None
 
-    if uri.startswith("http:") or uri.startswith("https:"):
-        fn = os.path.join(
-            tempfile.mkdtemp(prefix="miniwdl_import_uri_"),
-            os.path.basename(parse.urlsplit(uri).path),
-        )
-        request.urlretrieve(uri, filename=fn)
-        with open(fn, "r") as infile:
-            return ReadSourceResult(infile.read(), uri)
-    elif importer and (
-        importer.pos.abspath.startswith("http:") or importer.pos.abspath.startswith("https:")
-    ):
-        assert not os.path.isabs(uri), "absolute import from downloaded WDL"
-        return await read_source(parse.urljoin(importer.pos.abspath, uri), [], importer)
-    return await read_source_default(uri, path, importer)
+    async def read_source(uri, path, importer):
+        from urllib import parse, request
+
+        if uri.startswith("http:") or uri.startswith("https:"):
+            with tempfile.TemporaryDirectory(prefix="miniwdl_import_uri_") as tmpdir:
+                assert isinstance(tmpdir, str) and os.path.isdir(tmpdir)
+                fn = os.path.join(
+                    tmpdir,
+                    os.path.basename(parse.urlsplit(uri).path),
+                )
+                request.urlretrieve(uri, filename=fn)
+                with open(fn, "r") as infile:
+                    return ReadSourceResult(infile.read(), uri)
+        elif importer and (
+            importer.pos.abspath.startswith("http:") or importer.pos.abspath.startswith("https:")
+        ):
+            assert not os.path.isabs(uri), "absolute import from downloaded WDL"
+            return await read_source(parse.urljoin(importer.pos.abspath, uri), [], importer)
+        ans = await read_source_default(uri, path, importer)
+        if no_outside_imports:
+            # Require all imported local WDL files to be in/under the directory of the main WDL
+            # file (the first loaded), or one of the --path directoires.
+            nonlocal top_dir
+            if not top_dir:
+                top_dir = os.path.dirname(ans.abspath)
+            if not next(
+                (p for p in ([top_dir] + path) if path_really_within(ans.abspath, p)), False
+            ):
+                raise PermissionError(
+                    "denied import from outside main WDL file's directory; "
+                    "strike --no-outside-imports or add to --path: " + os.path.dirname(ans.abspath)
+                )
+        return ans
+
+    return read_source
 
 
 def fill_run_subparser(subparsers):
@@ -473,6 +504,12 @@ def fill_run_subparser(subparsers):
         action="store_true",
         help="upon failure, print error information JSON to standard output (in addition to standard error logging)",
     )
+    group.add_argument(
+        "-o",
+        metavar="OUT.json",
+        dest="stdout_file",
+        help="write JSON output/error to specified file instead of standard output (implies --error-json)",
+    )
     group = run_parser.add_argument_group("logging")
     group.add_argument(
         "-v",
@@ -551,6 +588,8 @@ def runner(
     no_cache=False,
     error_json=False,
     log_json=False,
+    stdout_file=None,
+    no_outside_imports=False,
     **kwargs,
 ):
     # set up logging
@@ -651,7 +690,12 @@ def runner(
 
         try:
             # load WDL document
-            doc = load(uri, path or [], check_quant=check_quant, read_source=read_source)
+            doc = load(
+                uri,
+                path or [],
+                check_quant=check_quant,
+                read_source=make_read_source(no_outside_imports),
+            )
 
             # parse and validate the provided inputs
             eff_root = (
@@ -669,12 +713,10 @@ def runner(
                 root=eff_root,  # if copy_input_files is set, then input files need not reside under the configured root
             )
         except Error.InputError as exn:
-            if error_json:
-                print(json.dumps(runtime.error_json(exn), indent=(None if log_json else 2)))
+            runner_standard_output(runtime.error_json(exn), stdout_file, error_json, log_json)
             die(exn.args[0])
         except Exception as exn:
-            if error_json:
-                print(json.dumps(runtime.error_json(exn), indent=(None if log_json else 2)))
+            runner_standard_output(runtime.error_json(exn), stdout_file, error_json, log_json)
             raise
 
         if json_only:
@@ -722,8 +764,7 @@ def runner(
         try:
             rundir, output_env = runtime.run(cfg, target, input_env, run_dir=run_dir, _cache=cache)
         except Exception as exn:
-            if error_json:
-                print(json.dumps(runtime.error_json(exn), indent=(None if log_json else 2)))
+            runner_standard_output(runtime.error_json(exn), stdout_file, error_json, log_json)
             exit_status = 2
             from_rundir = None
             while isinstance(exn, runtime.RunFailed):
@@ -762,7 +803,7 @@ def runner(
 
     # report
     outputs_json = {"outputs": values_to_json(output_env, namespace=target.name), "dir": rundir}
-    print(json.dumps(outputs_json, indent=(None if log_json else 2)))
+    runner_standard_output(outputs_json, stdout_file, error_json, log_json)
     return outputs_json
 
 
@@ -780,7 +821,11 @@ def runner_input_completer(prefix, parsed_args, **kwargs):
                 uri,
                 path=(parsed_args.path if hasattr(parsed_args, "path") else []),
                 check_quant=parsed_args.check_quant,
-                read_source=read_source,
+                read_source=make_read_source(
+                    parsed_args.no_outside_imports
+                    if hasattr(parsed_args, "no_outside_imports")
+                    else False
+                ),
             )
         except Exception as exn:
             argcomplete.warn(
@@ -973,7 +1018,7 @@ def runner_input_json_file(available_inputs, namespace, input_file, downloadable
     if input_file:
         input_file = input_file.strip()
     if input_file:
-        from ruamel.yaml import YAML  # delayed heavy import
+        import yaml  # delayed heavy import
 
         input_json = None
         if input_file[0] == "{":
@@ -983,10 +1028,10 @@ def runner_input_json_file(available_inputs, namespace, input_file, downloadable
         else:
             input_json = (
                 asyncio.get_event_loop()
-                .run_until_complete(read_source(input_file, [], None))
+                .run_until_complete(make_read_source(False)(input_file, [], None))
                 .source_text
             )
-        input_json = YAML(typ="safe", pure=True).load(input_json)
+        input_json = yaml.safe_load(input_json)
         try:
             ans = values_from_json(input_json, available_inputs, namespace=namespace)
         except Error.InputError as exn:
@@ -1158,6 +1203,19 @@ def validate_input_path(path, directory, downloadable, root):
     return path
 
 
+def runner_standard_output(content, stdout_file, error_json, log_json):
+    """
+    Write the runner output/error JSON in the way requested by the user
+    """
+    if error_json or stdout_file or "error" not in content:
+        content_json = json.dumps(content, indent=(None if log_json else 2))
+        if stdout_file:
+            with open(stdout_file, "w") as outfile:
+                print(content_json, file=outfile)
+        else:
+            print(content_json)
+
+
 def fill_run_self_test_subparser(subparsers):
     run_parser = subparsers.add_parser(
         "run_self_test", help="Run a short built-in workflow to test system configuration"
@@ -1370,6 +1428,7 @@ def localize(
     cfg=None,
     path=None,
     check_quant=True,
+    no_outside_imports=False,
     **kwargs,
 ):
     # set up logging
@@ -1416,7 +1475,12 @@ def localize(
 
         if infile:
             # load WDL document
-            doc = load(wdlfile, path or [], check_quant=check_quant, read_source=read_source)
+            doc = load(
+                wdlfile,
+                path or [],
+                check_quant=check_quant,
+                read_source=make_read_source(no_outside_imports),
+            )
 
             try:
                 target, input_env, input_json = runner_input(
