@@ -1031,6 +1031,9 @@ class TestWorkflowRunner(unittest.TestCase):
         self.assertTrue(os.path.isfile(outputs["stdout_txt"]))
         self.assertTrue(outputs["stdout_txt"].endswith(".txt"))
         self.assertFalse(outputs["stdout_txt"].endswith("stdout.txt"))
+        with open(outputs["stdout_txt"]) as stdout_txt:
+            stdout_lines = stdout_txt.read().strip().split("\n")
+            self.assertEquals(len(stdout_lines), 1)
         cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()), [])
         cfg.override({"file_io": {"delete_work": "failure"}, "task_runtime": {"_mock_interruptions": 2}})
         outputs = self._test_workflow(txt, cfg=cfg)
